--
--------------------------------------------------
--
-- This script optimizes the current database.
-- It also performs consistency checks and will
-- repair defective databases, as best it can.
--
-- These are the different levels:
--
--	0: Minor Optimizations (does not harm cursor operations)
--	1: Major Optimizations (will cause open cursors to fail)
--	2: Optimize & Shrink   (shrinking takes a *lot* of time)
--	3: Check, Optimize & Shrink
--	4: Check, Repair, Optimize & Shrink
--	5: Check, Repair (allowing data loss), Optimize & Shrink
--
-- WARNINGS:
--
--	Level 0 is the only safe level that can be run on a live database
--		This level is also safe for NavisionAnalysis databases
--
--	Level 1 can be run on a live database, but only if you time it right
--
--	Level 2 and up can't be called from other stored procedures (as these will be recompiled)
--
--	LEVEL 4 AND UP WILL SWITCH THE DATABASE INTO SINGLE-USER MODE
--
--------------------------------------------------
--
declare		@table		sysname
,		@column		sysname
,		@index		sysname
,		@statistic	sysname
,		@name		sysname
,		@statement	nvarchar(4000)
,		@level		integer
,		@master		integer
--
set		@level		=  0
set		@master		=  0	-- 0:customer, 1:master
--
--------------------------------------------------
--
if		@level		in (            4, 5)
begin
	select	@statement	= 'ALTER DATABASE ' + QUOTENAME(RTRIM(DB_NAME(0))) + ' SET SINGLE_USER'
	print	@statement
	exec	(@statement)
end
--
select		@statement	= 'DBCC CHECKCATALOG(' + QUOTENAME(RTRIM(DB_NAME(0))) + ') WITH NO_INFOMSGS'
print		@statement
exec		(@statement)
--
select		@statement	= 'DBCC CHECKDB(' + QUOTENAME(RTRIM(DB_NAME(0))) + (CASE @level WHEN 4 THEN ', REPAIR_REBUILD' WHEN 5 THEN ', REPAIR_ALLOW_DATA_LOSS' ELSE '' END) + ') WITH NO_INFOMSGS, TABLOCK'
print		@statement
exec		(@statement)
--
--------------------------------------------------
--
-- Specify the correct database options
--
--------------------------------------------------
--
if		db_name(0)	in (N'OSMaster', N'OSTdemo', N'OSTtest1', N'OSTtest2', N'OSTtest3', N'model')
begin
	set	@master		=  1	-- 0:customer, 1:master
end
--
if		@level		in (         3, 4, 5)
and		db_name(0)	in (N'OSCdb', N'OSTdb', N'OSMaster', N'OSTdemo', N'OSTtest1', N'OSTtest2', N'OSTtest3', N'StoreLogix', N'model')
begin
	select	@statement	= 'ALTER DATABASE ' + QUOTENAME(RTRIM(DB_NAME(0)))
				+ ' SET AUTO_CLOSE OFF'			-- a must-have for performance reasons (at least for OSCdb)
				+    ', AUTO_CREATE_STATISTICS ON'	-- a must-have for performance reasons
				+    ', AUTO_SHRINK OFF'		-- this is done every night by our maintenance plan
				+    ', AUTO_UPDATE_STATISTICS ON'	-- this is done every night as well, but it will help during heavy loads
				+    ', RECOVERY SIMPLE'		-- a must-have because we don't back-up the transaction log
	print	@statement
	exec	(@statement)
end
--
--------------------------------------------------
--
-- Rebuild (and thus reorganize and defrag) all indexes
--
--------------------------------------------------
--
if		@level		in (   1, 2, 3, 4, 5)
begin	--	DBCC DBREINDEX
		declare		TableCursor	scroll cursor for
				select		name
				from		sysobjects	as so
				where		OBJECTPROPERTY(so.id, 'IsUserTable')	=  1
				and		so.name					not like '#%'
				order by	so.name
		--
		open		TableCursor
		--
		while		1 = 1
		begin
				fetch next from	TableCursor
				into		@table
				if		@@fetch_status	<> 0	break
		
				select		@statement	= 'DBCC DBREINDEX(' + QUOTENAME(RTRIM(@table)) + ') WITH NO_INFOMSGS'
				print		@statement
				exec		(@statement)
		end
		--
		close		TableCursor
		deallocate	TableCursor
end
else	--	@level		=  0
begin	--	DBCC INDEXDEFRAG
		declare		IndexCursor	scroll cursor for
				select		so.name
				,		si.name
				from		sysindexes	as si
				inner join	sysobjects	as so
				on		si.id		=  so.id
				where		OBJECTPROPERTY(so.id, 'IsUserTable')				=  1
				and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsStatistics'), -1)	=  0
				and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IndexID'), -1)		<> 255	-- don't know why, but indexdefrag can't handle these
				and		so.name								not like '#%'
				order by	so.name									-- ordering by table is nice, but...
				,		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsClustered'), -1)	desc	-- it's important to handle the clustered index first
		--
		open		IndexCursor
		--
		while		1 = 1
		begin
				fetch next from	IndexCursor
				into		@table
				,		@index
				if		@@fetch_status	<> 0	break
		
				select		@statement	= 'DBCC INDEXDEFRAG(0, ' + QUOTENAME(RTRIM(@table)) + ', ' + QUOTENAME(RTRIM(@index)) + ') WITH NO_INFOMSGS'
				print		@statement
				execute		(@statement)
		end
		--
		close		IndexCursor
		deallocate	IndexCursor
end
--
--------------------------------------------------
--
-- Update usage data for all tables
--
--------------------------------------------------
--
if		@level		in (   1, 2, 3, 4, 5)
begin
		DBCC		UPDATEUSAGE(0)	WITH COUNT_ROWS
end
else	--	@level		=  0
begin
		DBCC		UPDATEUSAGE(0)	-- not that it wouldn't work..., but the COUNT_ROWS option just takes a *lot* of time
end
--
--------------------------------------------------
--
-- Remove all user-created statistics
--
--------------------------------------------------
--
if		@level		in (      2, 3, 4, 5)
and		@master		=  1
begin
		declare		StatCursor	scroll cursor for
				select distinct	so.name
				,		si.name
				from		sysindexes		as si
				inner join	sysobjects		as so
				on		si.id			= so.id
				where		OBJECTPROPERTY(so.id, 'IsUserTable')				=  1
				and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsStatistics'), -1)	=  1
				and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsAutoStatistics'), -1)	=  1
				and		so.name								not like '#%'
				order by	so.name
				,		si.name
		--
		open		StatCursor
		--
		while		1 = 1
		begin
				fetch next from	StatCursor
				into		@table
				,		@statistic
				if		@@fetch_status	<> 0	break
		
				select		@statement	= 'DROP STATISTICS ' + QUOTENAME(RTRIM(@table)) + '.' + QUOTENAME(RTRIM(@statistic))
				print		@statement
				execute		(@statement)
		end
		--
		close		StatCursor
		deallocate	StatCursor
end
--
--------------------------------------------------
--
-- Create statistics on all colums used in indices
--
--------------------------------------------------
--
if		@level		in (         3, 4, 5)
begin
	--	Begin by (re)renaming '[table].[index]' to 'index'
		declare		IndexCursor	scroll cursor for
				select distinct	so.name		as [Table]
				,		si.name		as [Index]
				from		sysobjects	as so
				inner join	sysindexes	as si
				on		so.id		=  si.id
				where		si.name		like '[[]%].[[]%]'
				and		OBJECTPROPERTY(so.id, 'IsUserTable')				=  1
				and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsStatistics'), -1)	<> 1
				and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsAutoStatistics'), -1)	<> 1
				order by	so.name
				,		si.name
		--
		open		IndexCursor
		--
		while		1 = 1
		begin
				fetch next from	IndexCursor
				into		@table
				,		@index
				if		@@fetch_status	<> 0	break
		
				select		@statement	=  'exec sp_rename @objname='''+quotename(rtrim(@table))+'.'+quotename(rtrim(@index))+''''
								+		', @newname='''+reverse(left(replace(replace(rtrim(reverse(@index)),'[',''),']',''),charindex('.',replace(replace(rtrim(reverse(@index)),'[',''),']',''))-1))+''''
								+		', @objtype=''INDEX'''
				print		@statement
				execute		(@statement)
		end
		--
		close		IndexCursor
		deallocate	IndexCursor
		--
		if		db_name(0)	=  N'OSCdb'
		begin	--	StoreCenter might still have improperly named indices, but these can only be renamed in recent versions
				if not exists	(select * from OESI_Param where ParamName='OESI_Version' and ParamValue in ('2002B','2002B+','2002C','2003D','2003D+','2004E'))	-- 'Exml', 'F', ... are okay
				begin	--	Rename improperly named indices
						declare		IndexCursor	scroll cursor for
								select distinct	so.name		as [Table]
								,		sc.name		as [Column]
								,		si.name		as [Index]
								from		sysobjects	as so
								inner join	sysindexes	as si
								on		so.id		=  si.id
								inner join	syscolumns	as sc
								on		si.name		=  sc.name
								where		OBJECTPROPERTY(so.id, 'IsUserTable')				=  1
								and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsStatistics'), -1)	<> 1
								and		ISNULL(INDEXPROPERTY(so.id, si.name, 'IsAutoStatistics'), -1)	<> 1
								order by	so.name
								,		sc.name
						--
						open		IndexCursor
						--
						while		1 = 1
						begin
								fetch next from	IndexCursor
								into		@table
								,		@column
								,		@index
								if		@@fetch_status	<> 0	break
						
								select		@statement	=  'sp_rename @objname='''+quotename(rtrim(@table))+'.'+quotename(rtrim(@index))+''''
												+          ', @newname=''X'+rtrim(@table)+'_'+rtrim(@column)+''''
												+          ', @objtype=''INDEX'''
								print		@statement
								execute		(@statement)
						end
						--
						close		IndexCursor
						deallocate	IndexCursor
						--
						exec		sp_createstats	@indexonly='indexonly'	-- For this to work an index should't use an existing column's name
				end
		end
		else
		begin
				exec		sp_createstats	@indexonly='indexonly'	-- For this to work an index should't use an existing column's name
		end
end
else	--	@level		in (0, 1, 2)
begin
				exec		sp_createstats	@indexonly='indexonly'	-- For this to work an index should't use an existing column's name
end
--
--------------------------------------------------
--
-- Update all statistics (== sp_updatestats)
--
--------------------------------------------------
--
if		@level		in (0, 1, 2, 3, 4, 5)
begin
		declare		TableCursor	scroll cursor for
				select		name
				from		sysobjects	as so
				where		OBJECTPROPERTY(so.id, 'IsUserTable')	=  1
				and		so.name					not like '#%'
				order by	name
		--
		open		TableCursor
		--
		while		1 = 1
		begin
				fetch next from	TableCursor
				into		@table
				if		@@fetch_status	<> 0	break

				select		@statement	= 'UPDATE STATISTICS ' + QUOTENAME(RTRIM(@table))
				print		@statement
				exec		(@statement)
		end
		--
		close		TableCursor
		deallocate	TableCursor
end
--
--------------------------------------------------
--
-- Recompile all stored procs and triggers
--
--------------------------------------------------
--
if		@level		in (      2, 3, 4, 5)
begin
		declare		SPTCursor	scroll cursor for
				select		name
				from		sysobjects	as so
				where	(	OBJECTPROPERTY(so.id, 'IsProcedure')	=  1
					or	OBJECTPROPERTY(so.id, 'IsTrigger')	=  1
					)
				and		name		not like '#%'
				order by	name
		--
		open		SPTCursor
		--
		while		1 = 1
		begin
				fetch next from	SPTCursor
				into		@name
				if		@@fetch_status	<> 0	break
		
				select		@statement = 'sp_recompile @objname=' + QUOTENAME(RTRIM(@name))
				exec		(@statement)
		end
		--
		close		SPTCursor
		deallocate	SPTCursor
end
--
--------------------------------------------------
--
-- Recompile all views
--
--------------------------------------------------
--
if		@level		in (      2, 3, 4, 5)
and		db_name(0)	not in (N'master', N'WinWa')
begin	--	This won't work on the master database as sysobjects itself is a view
	--	This won't work on WinWa, as that database uses encrypted views, which can't be refreshed (no mention of that in the SQL Server documentation though).
		declare		VCursor		scroll cursor for
				select		name
				from		sysobjects	as so
				where		OBJECTPROPERTY(so.id, 'IsView')	=  1
				and		name				not like '#%'
				order by	name
		--
		open		VCursor
		--
		while		1 = 1
		begin
				fetch next from	VCursor
				into		@name
				if		@@fetch_status	<> 0	break
		
				select		@statement = 'EXEC SP_REFRESHVIEW @viewname=' + QUOTENAME(RTRIM(@name))
				print		@statement
				exec		(@statement)
		end
		--
		close		VCursor
		deallocate	VCursor
end
--
--------------------------------------------------
--
-- Shrink the database
--
--------------------------------------------------
--
if		@level		in (      2, 3, 4, 5)
and		@master		=  1
begin
	select	@statement	= 'DBCC SHRINKDATABASE(' + QUOTENAME(RTRIM(DB_NAME(0))) + ', 0)'
	print	@statement
	exec	(@statement)
end
--
--------------------------------------------------
--
-- Switch back to multi-user mode
--
--------------------------------------------------
--
if		@level		in (            4, 5)
begin
	select	@statement	= 'ALTER DATABASE ' + QUOTENAME(RTRIM(DB_NAME(0))) + ' SET MULTI_USER'
	print	@statement
	exec	(@statement)
end
--
--------------------------------------------------
--
-- Set SQL Server to use a minimum of 1/8th, and
-- a maximum of 1/2 the available physical memory.
-- This ensures enough memory is left for other
-- applications and/or services.
--
-- Only for pure dedicated SQL Server systems the
-- maximum can be increased to over the 50% mark.
--
--------------------------------------------------
--
if		@level		in (         3, 4, 5)
and		db_name()	= 'master'		-- it's a bit annoying when this bit is done every single time...
begin							-- ...and anyway...
--		USE		master			-- ...RECONFIGURE only works when run from the master database

		SET NOCOUNT	ON
		
		DECLARE		@PhysicalMemory		INT
		,		@MinServerMemory	INT
		,		@MaxServerMemory	INT
		
		CREATE TABLE	#MSVer
		(		[Index]			INT		NULL
		,		[Name]			NVARCHAR(32)	NULL
		,		[Internal_Value]	INT		NULL
		,		[Character_Value]	NVARCHAR(120)	NULL
		)
		
		INSERT INTO	#MSVer
		EXEC		master.dbo.xp_msver
		
		SELECT		@PhysicalMemory		= [Internal_Value]
		FROM		#MSVer
		WHERE		Name			= 'PhysicalMemory'
		
		DROP TABLE	#MSVer
		
		SET		@MinServerMemory	=   (@PhysicalMemory + 7) / 8		-- 1/8	-- MIN
		IF		@PhysicalMemory		<= 256
			SET	@MaxServerMemory	=  ((@PhysicalMemory + 7) / 8) * 4	-- 4/8	-- 128 & 256
		ELSE IF		@PhysicalMemory		<= 512
			SET	@MaxServerMemory	=  ((@PhysicalMemory + 7) / 8) * 5	-- 5/8	-- 384 & 512
		ELSE	--	@PhysicalMemory		>  512
			SET	@MaxServerMemory	=  ((@PhysicalMemory + 7) / 8) * 6	-- 6/8	-- 768 & ...
		
		EXEC		master.dbo.sp_configure	@configname='show advanced option', @configvalue=1
		RECONFIGURE
		
		EXEC		master.dbo.sp_configure	@configname='min server memory', @configvalue=@MinServerMemory
		EXEC		master.dbo.sp_configure	@configname='max server memory', @configvalue=@MaxServerMemory
		RECONFIGURE
end
--
--------------------------------------------------
--
-- Finished
--
--------------------------------------------------
--
